package cn.newfeifan.mall.module.distri.dal.dataobject.orderpercentage;

import lombok.*;
import com.baomidou.mybatisplus.annotation.*;
import cn.newfeifan.mall.framework.mybatis.core.dataobject.BaseDO;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.constant.DistriConstants.GROSS_PROFIT_BONUS_QUOTA_PERC;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.GROSS_PROFIT_BONUS_QUOTA_PERC_EXCEED_MAXIMUM_LIMIT;

/**
 * 积分相关计算浮动百分比设置 DO
 *
 * @author 非繁人
 */
@TableName("distri_order_percentage")
@KeySequence("distri_order_percentage_seq") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OrderPercentageDO extends BaseDO {

    /**
     * 用户编号
     */
    @TableId
    private Long id;
    /**
     * 毛利 = 成本 * 0.38... (推广费用)
     */
    private String grossProfitPerc;
    /**
     * 推广-用户额度 百分比
     */
    private String grossProfitUserQuotaPerc;
    /**
     * 推广-推荐人额度 百分比
     */
    private String grossProfitAncestorQuotaPerc;
    /**
     * 推广-合赢奖额度(浮动) 百分比
     */
    private String grossProfitBonusQuotaPerc;
    /**
     * 推广-平台分成额度(浮动) 百分比
     */
    private String grossProfitPlatformQuotaPerc;
    /**
     * 分成百分比
     */
    private String divideIntoPerc;

    /**
     * 购物订单：直推人身价提升
     */
    private String orderAncestorSocialStatus;
    /**
     * 购物订单：本人身价提升
     */
    private String orderUserSocialStatus;
    /**
     * 新用户注册：直推人身价提升
     */
    private String registerAncestorSocialStatus;
    /**
     * 新用户注册：本人身价提升
     */
    private String registerSocialStatus;

    private Integer status;

    /**
     * 用户注册后，获得的基础最大合赢奖、直推奖积分限额
     */
    private String baseMaxQuota;

    /**
     * 收藏商品获取身价值，同一SPU商品收藏多次无效
     */
    private String collectSocialStatus;

    /**
     * 用户签到可获取的最大身价值
     */
    private String signInSocialStatusMax;
    /**
     * 签到获取身价
     */
    private String signInSocialStatus;

    public String getGrossProfitBonusQuotaPerc() {
        double grossProfitBonusQuotaPerc = Double.parseDouble(this.grossProfitBonusQuotaPerc);
        if(grossProfitBonusQuotaPerc > GROSS_PROFIT_BONUS_QUOTA_PERC){
            throw exception(GROSS_PROFIT_BONUS_QUOTA_PERC_EXCEED_MAXIMUM_LIMIT);
        }
        return this.grossProfitBonusQuotaPerc;
    }

}