package cn.newfeifan.mall.module.distri.service.sharepath;

import java.util.*;
import javax.validation.*;

import cn.newfeifan.mall.module.distri.controller.admin.sharepath.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.duser.DuserDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.ordercalc.OrderCalcDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.sharepath.SharePathDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.module.member.controller.admin.user.vo.MemberUserRespVO;

/**
 * 分销用户关系 Service 接口
 *
 * @author 非繁人
 */
public interface SharePathService {

    Long createSharePath(SharePathSaveReqVO createReqVO);

    void createSharePathByUserId(SharePathSaveReqVO createReqVO);

    void createSharePathByUserId(List<Long> userIds);


    /**
     * 更新分销用户关系
     *
     * @param updateReqVO 更新信息
     */
    void updateSharePath(@Valid SharePathSaveReqVO updateReqVO);

    /**
     * 删除分销用户关系
     *
     * @param id 编号
     */
    void deleteSharePath(Long id);

    /**
     * 获得分销用户关系
     *
     * @param id 编号
     * @return 分销用户关系
     */
    SharePathDO getSharePath(Long id);

    /**
     * 获得分销用户关系分页
     *
     * @param pageReqVO 分页查询
     * @return 分销用户关系分页
     */
    PageResult<SharePathDO> getSharePathPage(SharePathPageReqVO pageReqVO);




    /**
     * 转让分销用户关系
     *
     * @param sharePathtransFerenceRelaReqVO
     */
    void transferenceRela(SharePathtransFerenceRelaReqVO sharePathtransFerenceRelaReqVO);


    /**
     * 通过直推人, 获取当前的直推关系
     *
     * @param userId
     * @return
     */
    SharePathDO getSharePathByDescendant(Long userId);


    /**
     * 获取儿子订单计算分区
     *
     * @param userId
     * @return
     */
    List<OrderCalcDO> selectSons(Long userId, List<Long> userIds);

    /**
     * 获取兄弟订单分区
     *
     * @param userId
     * @param userId1
     * @return
     */
    List<OrderCalcDO> selectBrothers(Long userId, List<Long> userId1);


    Long queryParentBySonUserId(Long userId);

    List<DuserDO> selectSon(Long userId);


    List<Long> sonsId(Long userId);

    PageResult<SharePathRespVO> getDescendants(SharePathPageReqVO pageReqVO);
    TreeNode getTree(Long userId,  Integer maxDepth);

    TreeNode getTreeByNameOrMobile(SharePathTreeReqVO sharePathTreeReqVO);

    MemberUserRespVO getMemberUser(Long userId);
}