package cn.newfeifan.mall.module.trade.job.order;

import cn.newfeifan.mall.framework.quartz.core.handler.JobHandler;
import cn.newfeifan.mall.framework.tenant.core.job.TenantJob;
import cn.newfeifan.mall.module.distri.mq.message.order.CalcMessage;
import cn.newfeifan.mall.module.distri.mq.message.order.DistriOrderMessage;
import cn.newfeifan.mall.module.distri.mq.message.order.OrderCalcMessage;
import cn.newfeifan.mall.module.distri.mq.message.order.OrderItemMessage;
import cn.newfeifan.mall.module.trade.mq.producer.order.DistriOrderProducer;
import cn.newfeifan.mall.module.trade.service.order.TradeOrderQueryService;
import org.quartz.DisallowConcurrentExecution;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 交易订单自动计算 Job
 *
 * @author 非繁源码
 */
@Component
@DisallowConcurrentExecution
@Lazy(false)
public class TradeOrderAutoCalcJob implements JobHandler {

    @Resource
    private DistriOrderProducer distriOrderProducer;

    @Resource
    private TradeOrderQueryService tradeOrderQueryService;

    @Override
    @TenantJob
    public String execute(String param) {
        List<DistriOrderMessage> appTradeOrderDetailRespVos = tradeOrderQueryService.queryAllOrder();

        Map<Long, List<OrderItemMessage>> orderItemMessageGroupBy = appTradeOrderDetailRespVos.stream()
                .map(k -> OrderItemMessage.builder().price(k.getPrice())
                        .productCount(k.getProductCount())
                        .costPrice(k.getCostPrice())
                        .orderId(k.getOrderId())
                        .count(k.getCount())
                        .orderItemId(k.getOrderItemId())
                        .increaseIntegral(k.getIncreaseIntegral())
                        .ancestorIncreaseIntegral(k.getAncestorIncreaseIntegral())
                        .totalPrice(k.getTotalPrice()).build())
                .collect(Collectors.groupingBy(OrderItemMessage::getOrderId));

        List<OrderCalcMessage> orderCalcMessages = appTradeOrderDetailRespVos.stream().distinct().map(k -> {
            OrderCalcMessage orderCalcMessage = OrderCalcMessage.builder()
                    .orderId(k.getOrderId())
                    .name(k.getName())
                    .nickName(k.getNickName())
                    .no(k.getNo())
                    .merchantId(k.getMerchantId())
                    .productCount(k.getProductCount())
                    .shopId(k.getShopId())
                    .userId(k.getUserId()).build();
            List<OrderItemMessage> orderItemMessages = orderItemMessageGroupBy.get(k.getOrderId());
            orderCalcMessage.setOrderItemMessages(orderItemMessages);
            return orderCalcMessage;
        }).collect(Collectors.toList());


        CalcMessage calcMessage = CalcMessage.builder().orderCalcMessages(orderCalcMessages).distriOrderMessages(appTradeOrderDetailRespVos).build();
        distriOrderProducer.sendDistriOrderMessage(calcMessage);
        return "计算订单信息结束";


    }

}
